<?php

declare(strict_types=1);

namespace App\Controller;

use App\Repository\Post\PostRepository;
use App\Utils\Paginator\Paginator;
use Psr\Log\LoggerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Routing\Attribute\Route;

final class ListPostController extends AbstractController
{
    public function __construct(
        private readonly int $pageSize,
        private readonly int $maxVisiblePages,
        private readonly PostRepository $postRepository,
        private readonly Paginator $paginator,
        private readonly LoggerInterface $logger,
    ) {
    }

    #[Route('/', name: 'default')]
    public function default(): Response
    {
        return $this->forward(ListPostController::class . '::pageList', ['page' => 1]);
    }

    #[Route('/posts/{page}', name: 'page_list', requirements: ['page' => '\d+'], defaults: ['page' => 1])]
    public function pageList(int $page): Response
    {
        $totalPages = (int) ceil($this->postRepository->count([]) / $this->pageSize); /* TODO: tohle by si zaslouzilo asi cache */

        if ($totalPages === 0) {
            $this->logger->error('No posts found. Try run brilo:refresh-database command.');
            throw new NotFoundHttpException('No posts found.');
        }

        if ($page === 0) {
            $this->logger->error('Page number is zero. Redirecting to the first page.');
            return $this->redirectToRoute('page_list', ['page' => 1]);
        }

        // If page number is out of range, redirect to the last page.
        if ($page > $totalPages) {
            $this->logger->error(sprintf('Page number %d is out of range. Redirecting to the last page: %d', $page, $totalPages));
            return $this->redirectToRoute('page_list', ['page' => $totalPages]);
        }

        $posts = $this->postRepository->getPaginatedPosts($page, $this->pageSize);

        return $this->render('list_post/list.html.twig', [
            'posts' => $posts,
            'page' => $this->paginator->getVisiblePages($page, $totalPages, $this->maxVisiblePages),
        ]);
    }
}
