<?php

declare(strict_types=1);

namespace App\Service\Persist;

use App\Entity\Remote\Brilo\Comments\Comment;
use App\Repository\Comments\CommentsRepository;
use App\Repository\Post\PostRepository;
use Psr\Log\LoggerInterface;

class RemoteCommentConvertor
{
    public function __construct(
        protected readonly PostRepository $postRepository,
        protected readonly CommentsRepository $commentsRepository,
        protected readonly LoggerInterface $logger,
    ) {
    }

    /**
     * Converts remote entities into database entities
     * @param Comment[] $remoteComments
     * @return \Generator<\App\Entity\Database\Comments\Comment>
     */
    public function convert(iterable $remoteComments): \Generator
    {
        foreach ($remoteComments as $remoteComment) {
            $comment = $this->commentsRepository->find($remoteComment->id);
            if ($comment === null) {
                $this->logger->info("Creating new comment for ID: {$remoteComment->id}");
                yield $this->createComment($remoteComment);
            } else {
                $this->logger->info("Updating post for ID: {$remoteComment->id}");
                $this->updateComment($remoteComment, $comment);
                yield $comment;
            }
        }
    }

    /**
     * Create new comment entity from remote comment
     * @param Comment $remoteComment
     * @return \App\Entity\Database\Comments\Comment
     */
    protected function createComment(Comment $remoteComment): \App\Entity\Database\Comments\Comment
    {
        $post = $this->postRepository->find($remoteComment->postId) ?? throw new \LogicException("Post not found for ID: {$remoteComment->postId}");
        return new \App\Entity\Database\Comments\Comment(
            $remoteComment->id,
            $post,
            $remoteComment->name,
            $remoteComment->email,
            $remoteComment->body
        );
    }

    /**
     * Update existing comment entity with remote comment data
     * @param Comment $remoteComment
     * @param \App\Entity\Database\Comments\Comment $comment
     * @return void
     */
    protected function updateComment(Comment $remoteComment, \App\Entity\Database\Comments\Comment $comment): void
    {
        $post = $this->postRepository->find($remoteComment->postId) ?? throw new \LogicException("Post not found for ID: {$remoteComment->postId}");
        $comment->post = $post;
        $comment->name = $remoteComment->name;
        $comment->email = $remoteComment->email;
        $comment->body = $remoteComment->body;
    }
}
