<?php

declare(strict_types=1);

namespace App\Service\Remote;

use App\Entity\Remote\Brilo\Users\User;
use Psr\Log\LoggerInterface;
use Symfony\Component\HttpClient\Exception\TransportException;
use Symfony\Component\Serializer\Exception\MissingConstructorArgumentsException;
use Symfony\Component\Serializer\Exception\NotNormalizableValueException;
use Symfony\Component\Serializer\SerializerInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

trait BriloApiFetchTrait
{
    use RetryingClientTrait;

    protected readonly int $retryCount;
    protected readonly int $sleepTimeS;
    protected readonly LoggerInterface $logger;
    protected readonly HttpClientInterface $httpClient;
    protected readonly SerializerInterface $serializer;

    /**
     * Implementation of fetch logic. Handles retrying and logging of failed requests.
     * Returns deserialized data.
     * @template T
     *
     * @param string $uri Uri of the API endpoint.
     * @param class-string<T> $class Class name of the entity to deserialize data into.
     *
     * @return T[]
     * @throws \Exception
     */
    protected function fetchApiResponse(string $uri, string $class): array
    {
        return $this->retryingFailRequest($this->retryCount, $this->sleepTimeS, [
            TransportException::class, BriloRemoteApiException::class, NotNormalizableValueException::class, MissingConstructorArgumentsException::class
        ], $this->logger, function () use ($uri, $class) {
            $this->logger->debug('Trying to download brilo users');

            $response = $this->httpClient->request('GET', $uri);

            if ($response->getStatusCode() != 200) {
                $this->logger->error("Can't download brilo users. HTTP status code: " . $response->getStatusCode());
                throw new BriloRemoteApiException("Can't download brilo users. HTTP status code: " . $response->getStatusCode());
            }

            $this->logger->debug('Brilo users downloaded');

            $this->logger->debug('Deserialize brilo users');

            // extra attributes muzeme safe ignorovat
            return $this->serializer->deserialize($response->getContent(), $class . '[]', 'json');
        });
    }
}
